package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEvent();
            findCurrentEvent();
            findTodayEvent();
            findEventCreatedNow();
        } finally {
            entityManagerFactory.close();
        }
    }

    private static void findCurrentEvent() {
        System.out.println("-- current ongoing event  --");
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        CriteriaQuery<ScheduledEvent> query = criteriaBuilder.createQuery(ScheduledEvent.class);
        Root<ScheduledEvent> scheduledEventRoot = query.from(ScheduledEvent.class);
        Predicate scheduleDatePredicate = criteriaBuilder
                .equal(scheduledEventRoot.get(ScheduledEvent_.DATE), criteriaBuilder.currentDate());
        Predicate scheduleTimePredicate = criteriaBuilder
                .between(criteriaBuilder.currentTime(), scheduledEventRoot.get(ScheduledEvent_.start),
                        scheduledEventRoot.get(ScheduledEvent_.end));
        query.select(scheduledEventRoot)
             .where(criteriaBuilder.and(scheduleDatePredicate, scheduleTimePredicate));
        //The equivalent JPQL:
        //SELECT e FROM ScheduledEvent e WHERE CURRENT_DATE = e.date AND
        // CURRENT_TIME BETWEEN e.start AND e.end

        List<ScheduledEvent> resultList = entityManager.createQuery(query).getResultList();
        resultList.forEach(System.out::println);
        entityManager.close();
    }

    private static void findTodayEvent() {
        System.out.println("-- today event  --");
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        CriteriaQuery<ScheduledEvent> query = criteriaBuilder.createQuery(ScheduledEvent.class);
        Root<ScheduledEvent> scheduledEventRoot = query.from(ScheduledEvent.class);
        query.select(scheduledEventRoot)
             .where(criteriaBuilder.equal(
                     scheduledEventRoot.get(ScheduledEvent_.date),
                     criteriaBuilder.currentDate()
                     )
             );

        List<ScheduledEvent> resultList = entityManager.createQuery(query).getResultList();
        resultList.forEach(System.out::println);
        entityManager.close();
        //The equivalent JPQL:
        //SELECT e FROM ScheduledEvent e WHERE CURRENT_DATE = e.date
    }

    private static void findEventCreatedNow() {
        System.out.println("-- event created within 10 sec --");
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        CriteriaQuery<ScheduledEvent> query = criteriaBuilder.createQuery(ScheduledEvent.class);
        Root<ScheduledEvent> scheduledEventRoot = query.from(ScheduledEvent.class);
        query.select(scheduledEventRoot)
             .where(criteriaBuilder.between(
                     scheduledEventRoot.get(ScheduledEvent_.eventCreated),
                     criteriaBuilder.literal(Timestamp.valueOf(LocalDateTime.now().minusSeconds(10))),
                     criteriaBuilder.currentTimestamp())
             );
        //The equivalent JPQL:
        //SELECT e FROM ScheduledEvent e WHERE e.eventCreated
        // BETWEEN :timestampStart AND CURRENT_TIMESTAMP
        // where timestampStart = Timestamp.valueOf(LocalDateTime.now().minusSeconds(10)))

        List<ScheduledEvent> resultList = entityManager.createQuery(query).getResultList();
        resultList.forEach(System.out::println);
        entityManager.close();
    }

    private static void persistEvent() {
        ScheduledEvent e1 = ScheduledEvent.create("Test event",
                LocalDate.of(2018, 6, 1),
                LocalTime.of(9, 30, 45),
                LocalTime.of(11, 45, 30),
                LocalDateTime.now().minusDays(10));

        ScheduledEvent e2 = ScheduledEvent.create("Test event2",
                LocalDate.now(),
                LocalTime.of(9, 30, 45),
                LocalTime.of(23, 45, 30),
                LocalDateTime.now().minusMonths(1));

        ScheduledEvent e3 = ScheduledEvent.create("Test event3",
                LocalDate.now(),
                LocalTime.of(18, 0, 0),
                LocalTime.of(20, 0, 0),
                LocalDateTime.now());

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(e1);
        em.persist(e2);
        em.persist(e3);
        em.getTransaction().commit();

        System.out.println("-- all persisted entities --");
        em.createQuery("SELECT e FROM ScheduledEvent e")
          .getResultList()
          .forEach(System.out::println);
        em.close();
    }
}